package com.welnz.device.example

import androidx.compose.animation.core.LinearEasing
import androidx.compose.animation.core.animateFloat
import androidx.compose.animation.core.infiniteRepeatable
import androidx.compose.animation.core.rememberInfiniteTransition
import androidx.compose.animation.core.tween
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.IntrinsicSize
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Refresh
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.welnz.device.example.ui.models.IHomeViewModel
import com.welnz.device.example.ui.models.MockHomeViewModel
import com.welnz.device.example.ui.theme.ExampleTheme

@Preview(showBackground = false)
@Composable
fun HomeScreen(
    vm: IHomeViewModel = MockHomeViewModel()
) {
    val infiniteTransition = rememberInfiniteTransition()
    val rotation by infiniteTransition.animateFloat(
        initialValue = 0F,
        targetValue = 360F,
        animationSpec = infiniteRepeatable(
            animation = tween(2000, easing = LinearEasing)
        )
    )

    ExampleTheme {
        // A surface container using the 'background' color from the theme
        Surface(
            modifier = Modifier
                .fillMaxSize()
                .padding(all = 10.dp),
            color = MaterialTheme.colorScheme.background
        ) {
            Column(
                Modifier.width(IntrinsicSize.Max),
            ) {
                Column (
                    Modifier
                        .fillMaxWidth()
                        .padding(top = 25.dp, bottom = 25.dp),
                    horizontalAlignment = Alignment.CenterHorizontally
                ) {
                    Text(
                        text = "Willowbank Electronics Ltd",
                        fontSize = 24.sp,
                        textAlign = TextAlign.Center,
                        fontWeight = FontWeight.Bold,
                    )
                    Text(
                        text = "Example App",
                        fontSize = 16.sp,
                        textAlign = TextAlign.Center,
                    )
                }

                Column(
                    Modifier
                        .verticalScroll(rememberScrollState())
                        .fillMaxWidth()
                        .weight(1f)
                ) {
                    vm.devices.forEach{
                        DeviceRow(
                            device = it,
                            onConnectClicked = { vm.connect(it) }
                        )
                    }
                }

                Column(
                    Modifier
                        .fillMaxWidth()
                ) {
                    if (!vm.isScanning) {
                        QuickLargeIconButton(
                            icon = Icons.Filled.Refresh,
                            text = "Start Scan",
                            onClick = { vm.startScan() },
                            modifier = Modifier.fillMaxWidth()
                        )
                    } else {
                        QuickLargeIconButton(
                            icon = Icons.Filled.Refresh,
                            text = "Stop Scan",
                            onClick = { vm.stopScan() },

                            modifier = Modifier.fillMaxWidth(),
                            iconModifier = Modifier.graphicsLayer { rotationZ = rotation },
                        )
                    }
                }
            }
        }
    }
}
