package com.welnz.device.example.ui.models

import android.Manifest
import androidx.activity.ComponentActivity
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import com.welnz.device.example.PERMISSIONS_RATIONALE
import com.welnz.device.example.PERMISSION_REQUEST_LOCATION
import com.welnz.device.library.bluetooth.interfaces.SftDevice
import com.welnz.device.library.bluetooth.interfaces.WelManager
import com.welnz.device.library.enums.LogType
import com.welnz.device.library.bluetooth.WelManagerBuilder
import pub.devrel.easypermissions.EasyPermissions
import java.time.Instant

class HomeViewModel(private val parent: ComponentActivity) : IHomeViewModel {
    override var manager: WelManager = WelManagerBuilder()
        .setDelegate(this)
        .setContext(parent.applicationContext)
        .build();

    override var devices = mutableStateListOf<IDeviceViewModel>()
    override var deviceViewModel by mutableStateOf<IDeviceViewModel?>(null)

    override var lastMessage by mutableStateOf("")
    override var isScanning by mutableStateOf(false)

    override fun startScan() {
        val perms = arrayOf(Manifest.permission.ACCESS_FINE_LOCATION)
        if (!EasyPermissions.hasPermissions(parent.applicationContext, *perms)) {
            EasyPermissions.requestPermissions(
                parent,
                PERMISSIONS_RATIONALE,
                PERMISSION_REQUEST_LOCATION,
                *perms
            )
        }

        manager.startScan()
        isScanning = manager.isScanning
    }

    override fun stopScan() {
        manager.stopScan()
        isScanning = manager.isScanning
    }

    override fun connect(deviceModel: IDeviceViewModel) {
        if (deviceModel.device is SftDevice) {
            stopScan()

            deviceViewModel = DeviceViewModel(this, deviceModel.device)
            deviceModel.connect()
        }
    }

    override fun onLog(timestamp: Instant?, type: LogType?, message: String?) {
        lastMessage = message ?: ""
    }

    override fun onSftDeviceFound(device: SftDevice?) {
        if (device != null) {
            devices.add(DeviceViewModel(this, device))
        }
    }

    fun removeDevice(deviceModel: IDeviceViewModel) {
        deviceViewModel = null
        startScan()
    }
}